from sepmachine.capture.base import BaseCapture
from sepmachine.handler.base import BaseHandler

import typing
import tempfile
import os
import time
from loguru import logger


class BasePipeline(object):
    def __init__(self, capture: BaseCapture, handler: BaseHandler, extras: dict = None):
        self.capture: BaseCapture = capture
        self.handler: BaseHandler = handler

        # bind extra args
        self.extras: dict = extras
        self.capture.extras = extras
        self.handler.extras = extras

        logger.info(f"capture: {self.capture.__class__}")
        logger.info(f"handler: {self.handler.__class__}")
        logger.info(f"extras: {self.extras}")

    def run(self, video_path: str = None) -> bool:
        start_time = time.time()
        use_temp: bool = False
        if not video_path:
            logger.info("no specific video path, use temp file")
            use_temp = True
            video_path = tempfile.NamedTemporaryFile(suffix=".mp4", delete=False).name

        try:
            logger.info("start pipeline")
            logger.info(f"video: {video_path}")

            # capture
            self.capture.start(video_path)
            logger.info("start recording")
            self.capture.operate()
            logger.info("stop recording")
            capture_result: bool = self.capture.end()
            assert capture_result, "capture error"

            # handler
            handle_result: bool = self.handler.handle(video_path)
            assert handle_result, "handler error"
            logger.info("end pipeline")
            return True
        except Exception as e:
            logger.error(e)
            return False
        finally:
            if use_temp:
                try:
                    logger.debug(f"remove tempfile: {video_path}")
                    os.remove(video_path)
                # windows issue
                except PermissionError as e:
                    logger.error(e)
            end_time = time.time()
            cost = end_time - start_time
            logger.info(f"time cost: {cost}")

    def loop_run(self, video_path: str = None, loop_num: int = 1) -> typing.List[bool]:
        ret_list = list()
        for index in range(loop_num):
            logger.info(f"loop: {index}")
            ret = self.run(video_path)
            ret_list.append(ret)
        return ret_list



from sepmachine.handler.base import BaseHandler

from stagesepx.cutter import VideoCutter
from stagesepx.classifier.keras import KerasClassifier
from stagesepx.reporter import Reporter
from stagesepx.video import VideoObject
from loguru import logger


class KerasHandler(BaseHandler):
    def __init__(self, model_path: str = None, *args, **kwargs):
        self.model_path: str = model_path
        super(KerasHandler, self).__init__(*args, **kwargs)

    def handle(self, video_path: str) -> bool:
        super(KerasHandler, self).handle(video_path)
        video = VideoObject(video_path)
        if self.preload:
            video.load_frames()

        # --- cutter ---
        cutter = VideoCutter()
        res = cutter.cut(video)
        stable, unstable = res.get_range(threshold=0.98, offset=3)

        # --- classify ---
        cl = KerasClassifier()
        if self.model_path:
            logger.info("load existed pre-train model")
            cl.load_model(self.model_path)
        else:
            data_home = res.pick_and_save(
                stable, self.frame_count, to_dir=self.result_path
            )
            cl.train(data_home)
        self.classifier_result = cl.classify(video, stable)

        # --- draw ---
        r = Reporter()
        r.draw(self.classifier_result, report_path=self.result_report_path)
        return True



from sepmachine.capture.base import BaseCapture

import typing
import time
import ffmpeg
import tempfile
import os
import shutil
import subprocess
import signal
import sys
from loguru import logger
from minadb import ADBDevice


class ScrcpyCapture(BaseCapture):
    def __init__(self, serial_no: str = None, fps: int = 60):
        # args
        self.serial_no: str = serial_no
        self.fps: int = fps

        # others
        self.device: ADBDevice = ADBDevice(self.serial_no)
        self.record_stop: typing.Optional[typing.Callable] = None
        self.video_path: str = ""
        self.temp_video_path: str = ""
        logger.info(f"config: {self.__dict__}")

    def start(self, video_path: str) -> bool:
        self.video_path = video_path
        # save to temp file (scrcpy will save video on pc side directly)
        temp_video = tempfile.NamedTemporaryFile(mode="wb+", suffix=".mkv")
        temp_video_path = temp_video.name
        self.temp_video_path = temp_video_path
        temp_video.close()
        logger.debug(f"video will be saved to {self.temp_video_path}")

        device_flag = ["-s", self.serial_no] if self.serial_no else []
        record_command = [
            "scrcpy",
            *device_flag,
            "--render-expired-frames",
            "-Nr",
            self.temp_video_path,
        ]
        logger.info(f"start record: {record_command}")
        proc = subprocess.Popen(record_command)
        time.sleep(5)
        assert proc.poll() is None, f"run command failed: {record_command}"

        def stop():
            if sys.platform == "win32":
                self.device.kill_process_by_name("app_process")
                proc.terminate()
            else:
                proc.send_signal(signal.SIGINT)

        self.record_stop = stop

        return True

    def operate(self) -> bool:
        time.sleep(5)
        return True

    def end(self) -> bool:
        assert self.record_stop and self.video_path
        self.record_stop()

        try:
            # ffmpeg converter
            stream = ffmpeg.input(self.temp_video_path)
            stream = ffmpeg.filter(stream, "fps", fps=self.fps)
            stream = ffmpeg.output(stream, self.video_path)
            ffmpeg.run(stream, overwrite_output=True)
            logger.info(f"video convert finished. fps: {self.fps}")
        except FileNotFoundError:
            logger.warning("no ffmpeg installation found, skip fps converter")
            logger.warning("WARNING: ffmpeg is necessary for accuracy")
            shutil.copyfile(self.temp_video_path, self.video_path)
        finally:
            logger.info(f"video has been moved to: {self.video_path}")

        try:
            os.remove(self.temp_video_path)
            logger.debug(f"removed: {self.temp_video_path}")
        except PermissionError as e:
            logger.error(e)
            logger.warning("skip removing temp file")

        return True



from sepmachine.capture.base import BaseCapture

import typing
import time
import ffmpeg
import tempfile
import os
import shutil
from loguru import logger
from minadb import ADBDevice


class AdbCapture(BaseCapture):
    def __init__(self, serial_no: str = None, fps: int = 60):
        # args
        self.serial_no: str = serial_no
        self.fps: int = fps

        # others
        self.device: ADBDevice = ADBDevice(self.serial_no)
        self.record_stop: typing.Optional[typing.Callable] = None
        self.video_path: str = ""
        logger.info(f"capture config: {self.__dict__}")

    def start(self, video_path: str) -> bool:
        self.record_stop = self.device.screen_record()
        self.video_path = video_path
        return True

    def operate(self) -> bool:
        time.sleep(5)
        return True

    def end(self) -> bool:
        assert self.record_stop and self.video_path

        # save to temp file (for ffmpeg)
        temp_video = tempfile.NamedTemporaryFile(
            mode="wb+", suffix=".mp4", delete=False
        )
        temp_video_path = temp_video.name
        temp_video.close()
        self.record_stop(temp_video_path)
        logger.info(f"video saved to {temp_video_path}")

        try:
            # ffmpeg converter
            stream = ffmpeg.input(temp_video_path)
            stream = ffmpeg.filter(stream, "fps", fps=self.fps)
            stream = ffmpeg.output(stream, self.video_path)
            ffmpeg.run(stream, overwrite_output=True)
            logger.info(f"video convert finished. fps: {self.fps}")
        except FileNotFoundError:
            logger.warning("no ffmpeg installation found, skip fps converter")
            logger.warning("WARNING: ffmpeg is necessary for accuracy")
            shutil.copyfile(temp_video_path, self.video_path)
        finally:
            logger.info(f"video has been moved to: {self.video_path}")

        # remove temp file
        try:
            os.remove(temp_video_path)
            logger.debug(f"removed: {temp_video_path}")
        except PermissionError as e:
            logger.error(e)
            logger.warning("skip removing temp file")

        return True



import typing
import os
from loguru import logger
from stagesepx.classifier import ClassifierResult
from stagesepx import toolbox


class BaseHandler(object):
    def __init__(
        self, preload: bool = None, frame_count: int = None, result_path: str = None
    ):
        # config here
        self.preload: bool = preload or True
        self.frame_count: int = frame_count or 5
        # result
        self.classifier_result: typing.Optional[ClassifierResult] = None
        self.result_path: str = result_path
        self.result_report_path: str = ""
        if not self.result_path:
            self.result_path = toolbox.get_timestamp_str()
        os.makedirs(self.result_path, exist_ok=True)

        # extra args
        self.extras = dict()

        logger.info(f"handler config: {self.__dict__}")

    def handle(self, video_path: str) -> bool:
        self.result_report_path: str = os.path.join(
            self.result_path, f"{toolbox.get_timestamp_str()}.html"
        )
        return True

    def time_cost_between(self, start_stage: str , end_stage: str) -> float:
        return self.classifier_result.last(start_stage).timestamp - self.classifier_result.first(end_stage).timestamp



from sepmachine.capture.base import BaseCapture


class EmptyCapture(BaseCapture):
    """
    empty capture if you are going to send a video directly
    """

    def start(self, video_path: str) -> bool:
        return True

    def operate(self) -> bool:
        return True

    def end(self) -> bool:
        return True



from sepmachine.handler.base import BaseHandler

from stagesepx.cutter import VideoCutter
from stagesepx.classifier import SVMClassifier
from stagesepx.reporter import Reporter
from stagesepx.video import VideoObject


class NormalHandler(BaseHandler):
    def handle(self, video_path: str) -> bool:
        super(NormalHandler, self).handle(video_path)
        video = VideoObject(video_path)
        if self.preload:
            video.load_frames()

        # --- cutter ---
        cutter = VideoCutter()
        res = cutter.cut(video)
        stable, unstable = res.get_range(threshold=0.98, offset=3)
        data_home = res.pick_and_save(stable, self.frame_count, to_dir=self.result_path)

        # --- classify ---
        cl = SVMClassifier()
        cl.load(data_home)
        cl.train()
        self.classifier_result = cl.classify(video, stable)

        # --- draw ---
        r = Reporter()
        r.draw(self.classifier_result, report_path=self.result_report_path)
        return True



class BaseCapture(object):
    def __init__(self):
        self.extras = dict()

    def start(self, video_path: str) -> bool:
        raise NotImplementedError

    def operate(self) -> bool:
        raise NotImplementedError

    def end(self) -> bool:
        raise NotImplementedError
